#!/usr/bin/env python3
#
#===- clang-format-diff.py - ClangFormat Diff Reformatter ----*- python -*--===#
#
#                     The LLVM Compiler Infrastructure
#
# This file is distributed under the University of Illinois Open Source
# License. See LICENSE.TXT for details.
#
#===------------------------------------------------------------------------===#

#
# Slightly modified to handle the definition of functions, which do not
# require a space before the parenthesis
#

r"""
ClangFormat Diff Reformatter
============================

This script reads input from a unified diff and reformats all the changed
lines. This is useful to reformat all the lines touched by a specific patch.
Example usage for git/svn users:

  git diff -U0 --no-color HEAD^ | clang-format-diff.py -p1 -i
  svn diff --diff-cmd=diff -x-U0 | clang-format-diff.py -i

"""

import argparse
import difflib
import re
import subprocess
import sys
import tempfile
import os
from functools import reduce
try:
  from StringIO import StringIO
except ImportError:
  from io import StringIO

def main():
  parser = argparse.ArgumentParser(description=
                                   'Reformat changed lines in diff. Without -i '
                                   'option just output the diff that would be '
                                   'introduced. Use something like: '
                                   'git diff master..my-branch | ./sys/clang-format-diff.py -p1 -i')
  parser.add_argument('-i', action='store_true', default=False,
                      help='apply edits to files instead of displaying a diff')
  parser.add_argument('-p', metavar='NUM', default=0,
                      help='strip the smallest prefix containing P slashes')
  parser.add_argument('-regex', metavar='PATTERN', default=None,
                      help='custom pattern selecting file paths to reformat '
                      '(case sensitive, overrides -iregex)')
  parser.add_argument('-iregex', metavar='PATTERN', default=
                      r'.*\.(cpp|cc|c\+\+|cxx|c|cl|h|hpp|m|mm|inc|js|ts|proto'
                      r'|protodevel|java)',
                      help='custom pattern selecting file paths to reformat '
                      '(case insensitive, overridden by -regex)')
  parser.add_argument('-sort-includes', action='store_true', default=False,
                      help='let clang-format sort include blocks')
  parser.add_argument('-v', '--verbose', action='store_true',
                      help='be more verbose, ineffective without -i')
  parser.add_argument('--debug', action='store_true',
                      help='debug mode')
  parser.add_argument('-style',
                      help='formatting style to apply (LLVM, Google, Chromium, '
                      'Mozilla, WebKit)')
  parser.add_argument('-binary', default='clang-format',
                      help='location of binary to use for clang-format')
  args = parser.parse_args()

  def debug(s):
    if args.debug:
      sys.stderr.write(str(s) + '\n')

  # Extract changed lines for each file.
  filename = None
  lines_by_file = {}
  input = sys.stdin.read().split('\n')
  for lineidx, line in enumerate(input):
    match = re.search(r'^\+\+\+\ (.*?/){%s}(\S*)' % args.p, line)
    if match:
      filename = match.group(2)
    if filename is None:
      continue

    if args.regex is not None:
      if not re.match('^%s$' % args.regex, filename):
        continue
    else:
      if not re.match('^%s$' % args.iregex, filename, re.IGNORECASE):
        continue

    match = re.search(r'^@@.*\+(\d+)(,(\d+))?', line)
    if match:
      start_line = int(match.group(1))
      line_count = 1
      if match.group(3):
        line_count = int(match.group(3))
      if line_count == 0:
        continue
      range_start, range_end = None, None
      range_line = -1
      debug(line_count)
      i = 0
      while True:
        # stop iterating when finding the next diff
        if lineidx + i >= len(input):
          break

        # do not count lines that are removed
        if not input[lineidx + i].startswith('-'):
          range_line += 1

        if input[lineidx + i].startswith('+'):
          if range_start is None:
            range_start = start_line + range_line - 1
            debug('set range_start: ' + str(start_line + range_line))
        elif range_start is not None and range_end is None:
            range_end = start_line + range_line - 1
            debug('set range_end: ' + str(start_line + range_line))
            lines_by_file.setdefault(filename, []).append([range_start, range_end - 1])
            range_start, range_end = None, None

        debug('lineidx : ' + input[lineidx + i])

        if input[lineidx + i].startswith('diff') or (i != 0 and input[lineidx + i].startswith('@@')):
            break
        i += 1

  # Reformat files containing changes in place.
  for filename, lines in lines_by_file.items():
    debug('%s: %s' % (filename,lines))
    command = [args.binary, filename]
    if args.sort_includes:
      command.append('-sort-includes')
    if lines:
        s = [('-lines', str(x[0]) + ':' + str(x[1])) for x in lines]
        s = reduce(lambda x, y: x + y, s)
        command.extend(s)
    if args.style:
      command.extend(['-style', args.style])
    p = subprocess.Popen(command,
                         stdout=subprocess.PIPE,
                         stderr=None,
                         stdin=subprocess.PIPE,
                         universal_newlines=True)
    stdout, stderr = p.communicate()
    if p.returncode != 0:
      sys.exit(p.returncode)

    with open(filename) as f:
      code = f.readlines()
    formatted_code = StringIO(stdout).readlines()
    modified_lines = dict()
    if lines:
        for x in lines:
            for i in range(x[0], x[1] + 1):
                modified_lines[i] = True

    delta = 10
    # handle functions definitions/declarations: do not use space before (
    for i, l in enumerate(formatted_code):
        if modified_lines and not any(map(lambda x: x in modified_lines, range(i + 1 - delta, i + 1 + delta))):
            continue

        debug('formatted_code: ' + formatted_code[i])
        if formatted_code[i].startswith('R_API ') or formatted_code[i].startswith('static ') or formatted_code[i].startswith('R_IPI '):
            formatted_code[i] = formatted_code[i].replace(' (', '(')

        formatted_code[i] = formatted_code[i].replace('Elf_ (', 'Elf_(')

        while ' ? ' in formatted_code[i] and ' : ' in formatted_code[i]:
            pos_q = formatted_code[i].index(' ? ')
            pos_c = formatted_code[i].index(' : ')
            if pos_q >= pos_c:
                break
            formatted_code[i] = formatted_code[i].replace(' ? ', '? ', 1)
            formatted_code[i] = formatted_code[i].replace(' : ', ': ', 1)

    diff = difflib.unified_diff(code, formatted_code,
                                filename, filename,
                                '(before formatting)', '(after formatting)')
    diff_string = ''.join(diff)
    if len(diff_string) > 0:
      if args.i:
        f = tempfile.NamedTemporaryFile(delete=False)
        f.write(diff_string.encode())
        f.close()
        os.system('git apply -p0 < "%s"' % (f.name))
        os.unlink(f.name)
      else:
        sys.stdout.write(diff_string)

if __name__ == '__main__':
  main()



#!/usr/bin/env python2
# -*- coding: utf-8 -*-

"""
Example usage to regenerate traps.json:
    - open the dyld cache in r2 like this:
R_DYLDCACHE_FILTER=libsystem_kernel r2 -e bin.usextr=false ~/Library/Developer/Xcode/iOS\ DeviceSupport/12.1.2\ \(16C101\)\ arm64e/Symbols/System/Library/Caches/com.apple.dyld/dyld_shared_cache_arm64e

    - run the script with this command:
        #!pipe python2 /path/to/this/script.py > traps.json

"""

import r2pipe, json, re

r = r2pipe.open('#!pipe')

def walk_back_until (addr, pattern, min_addr):
    cursor = addr
    while cursor >= min_addr:
        op = r.cmdj('aoj@' + str(cursor))[0]['opcode']
        if re.search(pattern, op) != None:
            return cursor + 4
        if re.search(r'^ret', op) != None:
            return cursor + 4
        if re.search(r'^b ', op) != None:
            return cursor + 4
        cursor -= 4

    return min_addr

def carve_trap_num (addr, flag):
    saved_seek = r.cmd('?v $$')
    r.cmd('e io.cache=true')
    r.cmd('e emu.write=true')
    r.cmd('aei')
    r.cmd('aeim')
    min_addr = int(r.cmd('?v ' + flag), 0)
    emu_start = walk_back_until(addr - 4, r'^b|^ret|^invalid', min_addr)
    r.cmd('s ' + str(emu_start))
    obj = r.cmd('aefa 0x%08x~[0]:0' % addr)
    r.cmd('s ' + saved_seek)
    val = r.cmdj('pv4j@%s+0x14' % obj)['value']
    if val == 0:
        val = r.cmdj('pv4j@%s+0x18' % obj)['value']
    return val

def beautify_name (name):
    return re.sub(r'^_', '', name)

def carve_traps ():
    msgs = r.cmdj('axtj sym._mach_msg')
    if len(msgs) == 0:
        r.cmd('s sym._mach_msg')
        r.cmd('aae $SS @ $S')
        r.cmd('s-')
        msgs = r.cmdj('axtj sym._mach_msg')
        if len(msgs) == 0:
            print 'Cannot find refs to mach_msg!'
            return

    traps = {}
    for ref in msgs:
        if ref['type'] != 'CALL' or 'realname' not in ref:
            continue
        name = ref['realname']
        if re.search(r'^_mach_msg', name) != None:
            continue
        addr = ref['from']
        traps[addr] = {
            'name': name
        }

    result = []
    for addr in traps:
        trap = traps[addr]
        flag = 'sym.%s' % trap['name']
        trap['name'] = beautify_name(trap['name'])
        trap['num'] = carve_trap_num(addr, flag)
        if trap['num'] != None:
            result.append(trap)

    result.sort(key=lambda x: x['num'])

    return result

if __name__ == '__main__':
    traps = carve_traps()
    print json.dumps(traps, indent=4)



#!/usr/bin/env python2
# -*- coding: utf-8 -*-

import sys, re, json

header = """ /*
 * This file is generated in this way:
 *
 * python2 build_mig_index.py ~/xnu-4570.51.1/bsd/kern/trace_codes traps.json > mig_index.h
 *
 *
 * The traps.json file is generated from any dyld cache using the machtraps.py r2pipe script.
 *
 */
"""

def convert (trace_codes, trap_json):
    data = {}
    with open(trace_codes, 'r') as f:
        for line in f:
            splitted = re.compile('\s+').split(line.rstrip('\n'))
            name = splitted[1]
            code = int(splitted[0], 0)
            klass = code & 0xff000000
            if klass == 0xff000000: # MIG
                name = name.replace('MSG_', '')
                num = (code & 0x00ffffff) >> 2
                data[num] = name

    with open(trap_json, 'r') as f:
        traps = json.loads(f.read())
        for routine in traps:
            num = routine['num']
            if num in data:
                continue
            data[num] = routine['name']

    result = []
    for num in data:
        result.append((num, data[num]))

    result.sort(key = lambda x: x[0])

    print header
    print '#ifndef R_MIG_INDEX_H'
    print '#define R_MIG_INDEX_H\n'

    print '#define R_MIG_INDEX_LEN %d\n' % (len(data) * 2)

    print 'static const char * mig_index[R_MIG_INDEX_LEN] = {'
    for pair in result:
        print '\t"%d", "%s",' % pair
    print '};\n'

    print '#endif'


if __name__ == '__main__':
    if len(sys.argv) < 3:
        print 'usage %s bsd/kern/trace_codes traps.json' % sys.argv[0]
    else:
        convert(sys.argv[1], sys.argv[2])




#!/usr/bin/env python

""" Portable python script to read version from configure.acr """

import sys

with open(sys.argv[1], 'r') as f:
    for l in f:
        if 'SDBVER=' in l:
            version = l.strip('\n').split('=')[1]
            sys.stdout.write(version + '\n')



#!/usr/bin/env python3

import glob
import os
import sys

dirlist = [
    "binrz",
    "libr",
    "shlr/ar",
    "shlr/bochs",
    "shlr/gdb",
    "shlr/java",
    "shlr/ptrace-wrap",
    "shlr/qnx",
    "shlr/rar",
    "shlr/tcc",
    "shlr/winkd",
    "test/unit",
]

skiplist = [
    "/gnu/",
    "libr/anal/arch/vax/",
    "libr/asm/arch/riscv/",
    "libr/asm/arch/sh/gnu/",
    "libr/asm/arch/i8080/",
    "libr/asm/arch/z80/",
    "libr/asm/arch/avr/",
    "libr/arch/p/arm/aarch64/",
    "libr/hash/xxhash/",
    "libr/bin/mangling/cxx/",
    "libr/util/bdiff.c",
]

pattern = ["*.c", "*.cpp", "*.h", "*.hpp", "*.inc"]


def skip(filename):
    for s in skiplist:
        if s in filename:
            return True
    return False


try:
    for d in dirlist:
        print("Processing directory: {0}".format(d))
        for pat in pattern:
            print("Processing pattern: {0}".format(pat))
            for filename in glob.iglob(d + "/**/" + pat, recursive=True):
                if not skip(filename):
                    CMD = "clang-format -style=file -i {0}".format(filename)
                    print(CMD)
                    os.system(CMD)

except KeyboardInterrupt:
    print("clang-format.py interrupted by the user.")
    sys.exit(1)



#!/usr/bin/env python

""" Portable python script to read version from configure.acr """

import sys

full_version = False
no_newline = False

for arg in sys.argv[1:]:
    if arg == '-h' or arg == '--help':
        print('Only flag accepted here is --full-version')
        sys.exit(0)
    elif arg == '-n':
        no_newline = True
    elif arg == '--full-version':
        full_version = True
    else:
        print('Option %s not supported' % (arg,))
        sys.exit(1)
import os
r2root=os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
with open(os.path.join(r2root, 'configure.acr'), 'r') as f:
    f.readline()
    version = f.readline().split()[1]
    if no_newline:
        sys.stdout.write(version)
    else:
        sys.stdout.write(version + '\n')
    if full_version:
        versions = version.split('.')
        version_major = versions[0] if len(versions) > 0 else 0
        version_minor = versions[1] if len(versions) > 1 else 0
        version_patch = versions[2].replace('-git', '') if len(versions) > 2 else 0
        version_number = "%d%02d%02d"%(int(version_major), int(version_minor), int(version_patch))
        sys.stdout.write(version_major + '\n')
        sys.stdout.write(version_minor + '\n')
        sys.stdout.write(version_patch + '\n')
        sys.stdout.write(version_number + '\n')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
#
# rasm2 fuzzer
# ============
#
# Takes a binary executable as argument and uses its contents to feed
# the specified disassembler/assembler engines. Together with a reference
# engine(s), the input and outputs are chained through and any
# incompatibilities are turned into test cases which are written to stdout
# as streaming json.
#
# This program processes the disassembled instructions and groups them into
# similar "families", filtering out examples which are similar to those
# already reported, improving signal-to-noise ratio.
#
#
# History:
# v1 - Initial version
# v2 - Seperated asm/disasm engine constants
# v3 - fixed synchronization and added 3 level of canonical
#
# SchumBlubBlub - 2017
# Public Domain.

import os
import sh
import re
import json
from multiprocessing import Lock
from concurrent.futures import ProcessPoolExecutor
from binascii import hexlify

from sh import rasm2, dd

MAX_OPLEN = 20
MAX_METACASE_EXAMPLES = 1
MAX_META_META_CASE_EXAMPLES = 1
MARKER_NUMBER = "_NUM_"
MARKER_REGISTER = "_REG_"
MARKER_META_REGISTER = "_MREG_"
MARKER_WIDTH_MARKER = "_WIDTH_"
MARKER_SEGREG_MARKER = "_SEGREG_"
CONCURRENCY = 8
ASM_ENGINE = "x86.nz"  # See rasm2 -L. Note this script is not x86-specific
DISASM_ENGINE = "x86"  # capstone
REFERENCE_ASM_ENGINE = "x86.ks"
REFERENCE_DISASM_ENGINE = None  # no alternative right now

re_regs = re.compile(
    "al|ah|ax|eah|eax|rah|rax|bl|bh|bx|ebx|rbh|rbx|cl|ch|cx|ecx|rcx|dl|dh"
    "|dx|edl|edh|edx|rdx|si|esi|rsi|di|edi|rdi|sp|esp|rsp|bp|ebp|rbp|ip|eip"
    "|rip|r8|r9|r10|r11|r12|r13|r14|r15|r8d|r9d|r10d|r11d|r12d|r13d|r14d|"
    "r15d|r8w|r9w|r10w|r11w|r12w|r13w|r14w|r15w")
re_seg = re.compile("(cs|ds|es|fs|gs):")


def cannonical(s):
    return re.sub("0x[0-9a-fA-F]+|\d+", MARKER_NUMBER, s)


def meta_cannonical(s):
    s = cannonical(s)
    for r in re.findall("\[(.+?)]", s):
        r2 = re.sub("[a-z]+", MARKER_REGISTER, r)
        s = s.replace(r, r2)
    return s


def meta_meta_cannonical(s):
    s = meta_cannonical(s)
    s = re.sub('byte|word|dword|qword', MARKER_WIDTH_MARKER, s)
    for r in re_regs.findall(s):
        s = re.sub(r, MARKER_REGISTER, s)
    for r in re_seg.findall(s):
        s = re.sub(r, MARKER_SEGREG_MARKER, s)
    return s


def gen_testcase(cause, ins, inpairs, oins=""):
    inskey = cannonical(ins)
    insmkey = meta_cannonical(ins)
    insmmkey = meta_meta_cannonical(ins)
    return dict(
        cause=cause,
        ins=ins,
        inpairs=inpairs,
        oins=oins,
        case=inskey,
        metacase=insmkey,
        metametacase=insmmkey)


def check_hexpairs(orig_input_hexpairs):
    output = rasm2("-D", "-a", DISASM_ENGINE, orig_input_hexpairs)
    output = output.stdout.split(b"\n")[0].decode()
    output = re.split("\s+", output, 2)[-1]
    input_hexpairs, input_ins = re.split("\s+", output, 1)

    if input_ins == 'invalid':
        if REFERENCE_DISASM_ENGINE:
            coutput = rasm2(
                "-D", "-a", REFERENCE_DISASM_ENGINE, orig_input_hexpairs)
            coutput = coutput.stdout.split(b"\n")[0].decode()
            coutput = re.split("\s+", coutput, 2)[-1]
            cinput_hexpairs, cinput_ins = re.split("\s+", coutput, 1)
            if (cinput_ins != 'invalid'):
                print(cinput_hexpairs, cinput_ins)
                return gen_testcase("Disassemble False Fail", cinput_ins,
                                    cinput_hexpairs, input_ins)
        return

    try:
        output_hexpairs = rasm2(
            "-a", ASM_ENGINE, input_ins).stdout.split(b"\n")[0]
    except sh.ErrorReturnCode_1 as e:
        if "Cannot assemble" in str(e):
            return gen_testcase("Assemble False Fail", input_ins, input_hexpairs)
        return

    output_ins = rasm2("-d", "-a", DISASM_ENGINE, output_hexpairs)
    output_ins = output_ins.stdout.split(b"\n")[0].decode()

    if (input_ins != output_ins):
        return gen_testcase("Assemble != Dis+Assemble", input_ins,
                            input_hexpairs, output_ins)


def main():
    cases = {}
    meta_cases = {}
    meta_meta_cases = {}

    if len(sys.argv) != 2:
        print("Usage: %s <path to binary file>" % sys.argv[0])
        return 0

    if not os.path.exists(sys.argv[1]):
        print("No such file %s" % sys.argv[1])
        return 1

    fsize = os.stat(sys.argv[1]).st_size

    if fsize < MAX_OPLEN:
        print("muy pequeño:  %s" % sys.argv[1])
        return 1

    with open(sys.argv[1], "rb") as f:
        input_data = f.read()

    pool = ProcessPoolExecutor(CONCURRENCY)
    for offset in range(0, fsize-20, CONCURRENCY):
        inputs = [hexlify(input_data[o:o+MAX_OPLEN])
                  for o in range(offset, offset+CONCURRENCY)]
        tasks = pool.map(check_hexpairs, inputs)
        for res in tasks:
            if not res:
                continue
            inskey = res['case']
            insmkey = res['metacase']
            insmmkey = res['metametacase']
            meta_meta_cases[insmmkey] = meta_meta_cases.get(insmmkey, 0) + 1
            meta_cases[insmkey] = meta_cases.get(insmkey, 0) + 1
            if (meta_cases[insmkey] > MAX_METACASE_EXAMPLES or
                    meta_meta_cases[insmmkey] > MAX_META_META_CASE_EXAMPLES):
                pass
            elif inskey not in cases:
                cases[inskey] = cases.get(inskey, 0) + 1
                print("%s\n" % json.dumps(res, indent=4))

if __name__ == '__main__':
    import sys
    sys.exit(main())



#!/usr/bin/env python

""" Portable python script to convert Intel hex file to rapatch """

from intelhex import IntelHex
import argparse

parser = argparse.ArgumentParser(
    prog='ihex2rapatch',
    description='Convert Intel hex file to radare2 patch file')

parser.add_argument('source', help='Intel Hex source file')
parser.add_argument('target', help='radare2 patch target file')

args = parser.parse_args()

f = open(args.target, 'w')

ih = IntelHex(args.source)
for segment in ih.segments():
    f.write(hex(segment[0]) + ': ')
    for x in range(segment[0],segment[1]):
        f.write(f'{ih[x]:02x}')
    f.write('\n')

f.close()



#!/usr/bin/env python
# args [sdbpath] [input] [output]
import os
import sys
import subprocess


try:
	sdb_exe = sys.argv[1]
	_input = sys.argv[2]
	_output = sys.argv[3]
	_tmpfile = _input + ".tmp"
	with open(_input) as lines:
		res = ""
		for line in lines:
			# E.g. in linux-x86-64.sdb.txt:
			# accept=0x80,43,3,
			# And in linux-arm-64.sdb.txt:
			# accept=0,202
			kv = line.rstrip().split("=", 1)
			if kv[0] == "_":
				res += line
			else:
				vv = kv[1].split(",")
				res += vv[0] + "." + vv[1] + "=" + kv[0] + "\n"
				# Can't just append the original line, because
				# r_syscall_item_new_from_string splits it by commas
				# and wants at least 3 items in the result, whereas
				# original lines, at least in some archs, have only
				# two items. For compatibitity with gen.sh, always
				# have at least 4 items.
				vv.extend([ '' for i in range(4 - len(vv)) ])
				res += kv[0] + "=" + ",".join(vv) + "\n"
		with open(_tmpfile, "w") as file:
			file.write(res)
	subprocess.call([sdb_exe, _output, "==", _tmpfile])
except Exception as e:
	print(e)
	print("Usage: gen.py [sdb_exe] [input] [output]")




import sys

if len(sys.argv)!=2:
    print "Usage:"
    print "   First run 'dumpbin /exports your_file.dll/.lib > your_file_dumpbin.txt'"
    print "   Then run '%s your_file_dumpbin.txt > your_file.sdb.txt'" % (sys.argv[0],)
    print "   Note: this script will strip away function signatures, so overloaded C++ methods will all have the same name"
    sys.exit(1)

d = open(sys.argv[1]).read().split("\n")
for l in d:
    ls = l.split(None, 1)
    if len(ls)!=2:
        continue
    ordinal = ls[0]
    desc = ls[1]
    desc = desc.split("(", 1)
    if len(desc) <= 1:
        sys.stderr.write("Warning, skipping line: " + l + "\n")
        continue
    rawname = desc[0].strip()
    desc = desc[1].strip()
    if desc[-1] != ')':
        sys.stderr.write("Warning, skipping line: " + l + "\n")
        continue
    fulldesc = desc[:-1].strip()
    desc = fulldesc
    if desc.endswith('const'):
        desc = desc[:-len('const')]
    desc = desc.split('(', 1)
    pre = desc[0].strip()
    post = ""
    if len(desc)>1:
        desc = desc[1]
        desc = desc.split(')')
        if len(desc)>1:
            post = desc[-1].strip()
    if post != "":
        sys.stderr.write("Warning, skipping line: " + l + "\n")
        continue
    desc = pre
    i = len(desc) - 1
    deep = 0
    while i >= 0:
        if desc[i] == ">":
            deep += 1
        if desc[i] == "<":
            deep -= 1
        if desc[i] != " ":
            i -= 1
            continue
        if deep > 0:
            i -= 1
            continue
        break
    desc = desc[i+1:]
    print ordinal+"="+desc



from setuptools import setup
import os

def popen(x):
    res = os.popen(x).read()
    return res.strip() # decode("utf-8").strip()

with open("README.md") as fd:
    readme = fd.read()
r2_version = popen("../../configure -qV")

setup(
    name='radare2',
    version=r2_version,
    description="The UNIX friendly reverse engineering framework and toolchain",
    long_description=readme,
    long_description_content_type="text/markdown",
    author="pancake",
    author_email="pancake@nopcode.org",
    url="https://www.radare.org/",
    packages=[
        'radare2',
    ],
    install_requires=[
        'r2pipe',
        'r2env'
    ],
    entry_points = {
        'console_scripts': [
            'r2 = radare2.main:main'
        ]
    },
)



#!/usr/bin/env python
# Auto-generate radare syscall profile for OpenBSD from syscall.h
# (c) Edd Barrett 2011

import sys
import copy

f = open("/usr/include/sys/syscall.h", "r")

rec = { "name" : None, "args" : None }
recs = {}

for line in f:
    if line.startswith("/* syscall:"):
        # extract syscall name
        openq = line.find('"')
        closeq = line.find('"', openq + 1)
        rec["name"] = line[openq+1:closeq]

        # extract number of args
        args = line.find("args:")
        args = args + len("args: ")
        rec["args"] = line[args:].count('"') / 2
    elif line.startswith("#define"):

        if "SYS_MAXSYSCALL" in line:
            continue

        # extract syscall number
        sp = line.split("\t")
        callnum = sp[2].rstrip()

        # check required info is there
        for i in rec:
            if i == None:
                print("missing info for %s" % str(rec))
                sys.exit(1)
            
        recs[int(callnum)] = (copy.copy(rec))
        rec = { "name" : None, "args" : None }
f.close()


out = open("openbsd.c", "w")
out.write("#include \"r_syscall.h\"\n\n/* syscall-openbsd */\n")
out.write("RSyscallItem syscalls_openbsd_x86[] = {\n")

keys = recs.keys()
for call in keys:
    out.write("  { \"%s\", 0x80, %d, %d },\n" % 
            (recs[call]["name"], call, recs[call]["args"]))

out.write("};\n")
out.close()



import r2pipe
import json

def chk(x):
	if x[1]['opcode'] == 'svc 0x80':
		name = x[0]['flags'][0][8:]
		sysnum = int(x[0]['opcode'].split(' ')[2], 16)
		print ("%d\t%s"%(sysnum, name))

dev_pid = "23f88587e12c30376f8ab0b05236798fdfa4e853/4903"

r2 = r2pipe.open("frida://" + dev_pid)
print("Importing symbols from libSystem...")
r2.cmd(".=!i*")
r2.cmd(".=!ie* libSystem.B.dylib")
print("Finding syscalls...")
funcs = r2.cmd("pdj 2 @@ sym.fun.*")

for doc in funcs.split('\n'):
	if len(doc) > 1:
		chk(json.loads(doc))
r2.quit()
print("Thanks for waiting")



#!/usr/bin/env python
"""Meson build for radare2"""

import argparse
import glob
import logging
import os
import re
import shutil
import subprocess
import sys

BUILDDIR = 'build'
BACKENDS = ['ninja', 'vs2015', 'vs2017', 'vs2019']

PATH_FMT = {}
R2_PATH = {
    'R2_LIBDIR': r'lib',
    'R2_INCDIR': r'include',
    'R2_DATDIR': r'share',
    'R2_WWWROOT': r'{R2_DATDIR}\www',
    'R2_SDB': r'{R2_DATDIR}',
    'R2_ZIGNS': r'{R2_DATDIR}\zigns',
    'R2_THEMES': r'{R2_DATDIR}\cons',
    'R2_FORTUNES': r'{R2_DATDIR}\doc',
    'R2_FLAGS': r'{R2_DATDIR}\flag',
    'R2_HUD': r'{R2_DATDIR}\hud'
}

MESON = None
ROOT = None
log = None

def set_global_variables():
    """[R_API] Set global variables"""
    global log
    global ROOT
    global MESON

    ROOT = os.path.abspath(os.path.join(os.path.dirname(__file__), os.path.pardir))

    logging.basicConfig(format='[%(name)s][%(levelname)s]: %(message)s',
                        level=logging.DEBUG)
    log = logging.getLogger('r2-meson')

    with open(os.path.join(ROOT, 'configure.acr')) as f:
        f.readline()
        version = f.readline().split()[1].rstrip()

    if os.name == 'nt':
        meson = os.path.join(os.path.dirname(sys.executable), 'Scripts', 'meson.exe')
        if os.path.exists(meson):
            MESON = [meson]
        else:
            meson = os.path.join(os.path.dirname(sys.executable), 'Scripts', 'meson.py')
            MESON = [sys.executable, meson]
    else:
        MESON = ['meson']

    PATH_FMT['ROOT'] = ROOT
    PATH_FMT['R2_VERSION'] = version

    log.debug('Root: %s', ROOT)
    log.debug('Meson: %s', MESON)
    log.debug('Version: %s', version)

def meson(command, rootdir=None, builddir=None, prefix=None, backend=None,
          release=False, shared=None, *, options=[]):
    """[R_API] Invoke meson"""
    cmd = MESON + [command]
    if rootdir:
        cmd.append(rootdir)
    if builddir:
        cmd.append(builddir)
    if prefix:
        cmd.append('--prefix={}'.format(prefix))
    if backend:
        cmd.append('--backend={}'.format(backend))
    if release:
        cmd.append('--buildtype=release')
    if shared != None:
        cmd.append('--default-library={}'.format('shared' if shared else 'static'))
    if options:
        cmd.extend(options)

    log.debug('Invoking meson: %s', cmd)
    ret = subprocess.call(cmd)
    if ret != 0:
        log.error('Meson error. Exiting.')
        sys.exit(1)

def ninja(folder, *targets):
    """[R_API] Invoke ninja"""
    command = ['ninja', '-C', folder]
    if targets:
        command.extend(targets)
    log.debug('Invoking ninja: %s', command)
    ret = subprocess.call(command)
    if ret != 0:
        log.error('Ninja error. Exiting.')
        sys.exit(1)

def msbuild(project, *params):
    """[R_API] Invoke MSbuild"""
    command = ['msbuild', project]
    if params:
        command.extend(params)
    log.info('Invoking MSbuild: %s', command)
    ret = subprocess.call(command)
    if ret != 0:
        log.error('MSbuild error. Exiting.')
        sys.exit(1)

def copytree(src, dst, exclude=()):
    src = src.format(**PATH_FMT)
    dst = dst.format(**PATH_FMT).format(**PATH_FMT)
    log.debug('copytree "%s" -> "%s"', src, dst)
    shutil.copytree(src, dst, ignore=shutil.ignore_patterns(*exclude) if exclude else None)

def move(src, dst):
    src = src.format(**PATH_FMT)
    dst = dst.format(**PATH_FMT).format(**PATH_FMT)
    term = os.path.sep if os.path.isdir(dst) else ''
    log.debug('move "%s" -> "%s%s"', src, dst, term)
    for file in glob.iglob(src):
        shutil.move(file, dst)

def copy(src, dst):
    src = src.format(**PATH_FMT)
    dst = dst.format(**PATH_FMT).format(**PATH_FMT)
    term = os.path.sep if os.path.isdir(dst) else ''
    log.debug('copy "%s" -> "%s%s"', src, dst, term)
    for file in glob.iglob(src, recursive='**' in src):
        shutil.copy2(file, dst)

def makedirs(path):
    path = path.format(**PATH_FMT).format(**PATH_FMT)
    log.debug('makedirs "%s"', path)
    os.makedirs(path)

def xp_compat(builddir):
    log.info('Running XP compat script')

    with open(os.path.join(builddir, 'REGEN.vcxproj'), 'r') as f:
        version = re.search('<PlatformToolset>(.*)</PlatformToolset>', f.read()).group(1)

    if version.endswith('_xp'):
        log.info('Skipping %s', builddir)
        return

    log.debug('Translating from %s to %s_xp', version, version)
    newversion = version+'_xp'

    for f in glob.iglob(os.path.join(builddir, '**', '*.vcxproj'), recursive=True):
        with open(f, 'r') as proj:
            c = proj.read()
        c = c.replace(version, newversion)
        with open(f, 'w') as proj:
            proj.write(c)
            log.debug("%s .. OK", f)

def build(args):
    """ Build radare2 """
    log.info('Building radare2')
    r2_builddir = os.path.join(ROOT, args.dir)
    options = ['-D%s' % x for x in args.options]
    if args.webui:
        options.append('-Duse_webui=true')
    if args.local:
        options.append('-Dlocal=true')
    if args.fuzz:
        options.append('-Denable_libfuzzer=true')
    if not os.path.exists(r2_builddir):
        meson('setup', builddir=r2_builddir, prefix=args.prefix, backend=args.backend,
              release=args.release, shared=args.shared, options=options)
    if args.backend != 'ninja':
        # XP support was dropped in Visual Studio 2019 v142 platform
        if args.backend == 'vs2017' and args.xp:
            xp_compat(r2_builddir)
        if not args.project:
            project = os.path.join(r2_builddir, 'radare2.sln')
            params = ['/m', '/clp:Summary;Verbosity=minimal']
            if args.backend == 'vs2017' and args.xp:
                params.append('/p:XPDeprecationWarning=false')
            msbuild(project, *params)
    else:
        ninja(r2_builddir)

def install(args):
    """ Install radare2 """
    meson('install', options=['-C', '{}'.format(args.dir), '--no-rebuild'])

def main():
    # Create logger and get applications paths
    set_global_variables()

    # Create parser
    parser = argparse.ArgumentParser(description='Mesonbuild scripts for radare2')
    # --sanitize=address,signed-integer-overflow for faster build
    parser.add_argument('--sanitize', nargs='?',
            const='address,undefined,signed-integer-overflow', metavar='sanitizers',
            help='Build radare2 with sanitizer support (default: %(const)s)')
    parser.add_argument('--fuzz', action='store_true',
            help='Build radare2 with libFuzzer support')
    parser.add_argument('--project', action='store_true',
            help='Create a visual studio project and do not build.')
    parser.add_argument('--release', action='store_true',
            help='Set the build as Release (remove debug info)')
    parser.add_argument('--backend', choices=BACKENDS, default='ninja',
            help='Choose build backend (default: %(default)s)')
    parser.add_argument('--shared', action='store_true',
            help='Link dynamically (shared library) rather than statically')
    parser.add_argument('--local', action='store_true',
            help='Adds support for local/side-by-side installation (sets rpath if needed)')
    parser.add_argument('--prefix', default=None,
            help='Set project installation prefix')
    parser.add_argument('--dir', default=BUILDDIR, required=False,
            help='Destination build directory (default: %(default)s)')
    parser.add_argument('--alias', action='store_true',
            help='Show the "m" alias shell command')
    parser.add_argument('--xp', action='store_true',
            help='Adds support for Windows XP')
    parser.add_argument('--pull', action='store_true',
            help='git pull before building')
    parser.add_argument('--nosudo', action='store_true',
            help='Do not use sudo for install/symstall/uninstall')
    parser.add_argument('--uninstall', action='store_true',
            help='Uninstall')
    parser.add_argument('--symstall', action='store_true',
            help='Install using symlinks')
    parser.add_argument('--webui', action='store_true',
            help='Install WebUIs')
    parser.add_argument('--install', action='store_true',
            help='Install radare2 after building')
    parser.add_argument('--options', nargs='*', default=[])
    args = parser.parse_args()
    if args.alias:
        print("alias m=\"" + os.path.abspath(__file__) + "\"")
        sys.exit(0);
    if args.sanitize:
        if os.uname().sysname == 'OpenBSD':
            log.error("Sanitizers unsupported under OpenBSD")
            sys.exit(1)
        sanitizers = args.sanitize
        if args.fuzz and 'fuzzer' not in sanitizers:
            sanitizers = "fuzzer," + sanitizers
        cflags = os.environ.get('CFLAGS')
        if not cflags:
            cflags = ''
        os.environ['CFLAGS'] = cflags + ' -fsanitize=' + sanitizers
        if os.uname().sysname != 'Darwin':
          ldflags = os.environ.get('LDFLAGS')
          if not ldflags:
              ldflags = ''
          os.environ['LDFLAGS'] = ldflags + ' -fsanitize=' + sanitizers 

    # Check arguments
    if args.pull:
        # Attempt to update from an existing remote
        upstream_remote = subprocess.check_output(["git", "remote", "-v"]).decode()
        try:
            remote = re.search(r'(.*?)\t.*radareorg/radare2 \(fetch\)', upstream_remote).group(1)
        except AttributeError: # If search misses, None.group() throws this
            remote = 'https://github.com/radareorg/radare2'
        os.system(f'git pull {remote} master')
    if args.project and args.backend == 'ninja':
        log.error('--project is not compatible with --backend ninja')
        sys.exit(1)
    if args.xp and args.backend in 'ninja':
        log.error('--xp is not compatible with --backend ninja')
        sys.exit(1)
    if args.xp and args.backend in 'vs2019':
        log.error('--xp is not compatible with --backend vs2019')
        sys.exit(1)
    if not args.prefix:
        args.prefix = os.path.join(ROOT, args.dir, 'priv_install_dir')
    else:
        args.prefix = os.path.abspath(args.prefix)
    for option in args.options:
        if '=' not in option:
            log.error('Invalid option: %s', option)
            sys.exit(1)
        key, value = option.split('=', 1)
        key = key.upper()
        if key not in R2_PATH:
            continue
        if os.path.isabs(value):
            log.error('Relative path is required: %s', option)
            sys.exit(1)
        R2_PATH[key] = os.path.normpath(value)

    PATH_FMT.update(R2_PATH)

    sudo = 'sudo '
    if args.nosudo:
        sudo = ''
    # Build it!
    log.debug('Arguments: %s', args)
    build(args)
    if args.uninstall:
        os.system(sudo + 'make uninstall PWD="$PWD/build" BTOP="$PWD/build/binr"')
    if args.install:
        install(args)
    if args.symstall:
        os.system(sudo + 'make symstall PWD="$PWD/build" BTOP="$PWD/build/binr"')

if __name__ == '__main__':
    main()
